import { interval, animationFrameScheduler, of, Subject } from 'rxjs';
import { debounceTime, distinctUntilChanged, filter, map, mapTo, skipWhile, tap, switchMap } from 'rxjs/operators';
import { __extends, __decorate, __metadata } from 'tslib';
import { InjectionToken, Attribute, Directive, forwardRef, inject, Inject, Output, Input, Self, EventEmitter, SkipSelf, ContentChildren, Injectable, NgModule, Pipe, defineInjectable, ElementRef } from '@angular/core';

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * @param {?} param
 * @param {?} value
 * @param {?=} currentTime
 * @return {?}
 */
function processAudioParam(param, value, currentTime) {
    if (currentTime === void 0) { currentTime = 0; }
    if (param.cancelAndHoldAtTime) {
        param.cancelAndHoldAtTime(currentTime);
    }
    else {
        param.cancelScheduledValues(currentTime);
        param.setValueAtTime(guard(param.value), currentTime);
    }
    if (typeof value === 'number') {
        param.setValueAtTime(guard(value), currentTime);
        return;
    }
    if (value instanceof Array) {
        processSchedule(param, value, currentTime);
        return;
    }
    if (!('mode' in value)) {
        param.setValueCurveAtTime(value.value, currentTime, value.duration);
        return;
    }
    param.setValueAtTime(guard(param.value), currentTime);
    processAutomation(param, value, currentTime);
}
/**
 * @param {?} param
 * @param {?} value
 * @param {?} currentTime
 * @return {?}
 */
function processSchedule(param, value, currentTime) {
    value.forEach(function (automation) {
        if ('mode' in automation) {
            processAutomation(param, automation, currentTime);
        }
        else {
            param.setValueCurveAtTime(automation.value, currentTime, automation.duration);
        }
        currentTime += automation.duration;
    });
}
/**
 * @param {?} param
 * @param {?} __1
 * @param {?} currentTime
 * @return {?}
 */
function processAutomation(param, _a, currentTime) {
    var value = _a.value, _b = _a.mode, mode = _b === void 0 ? 'instant' : _b, duration = _a.duration;
    switch (mode) {
        case 'instant':
            param.setValueAtTime(guard(value), currentTime);
            param.setValueAtTime(guard(value), currentTime + duration);
            break;
        case 'exponential':
            if (value < 0 || value * param.value < 0) {
                param.linearRampToValueAtTime(guard(value), currentTime + duration);
            }
            else {
                param.exponentialRampToValueAtTime(guard(value), currentTime + duration);
            }
            param.setValueAtTime(guard(value), currentTime + duration);
            break;
        case 'linear':
            param.linearRampToValueAtTime(guard(value), currentTime + duration);
            break;
    }
}
/**
 * @param {?} v
 * @return {?}
 */
function guard(v) {
    return v || 0.00000001;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * @template K
 * @param {?=} param
 * @return {?}
 */
function audioParam(param) {
    if (param === void 0) { param = ''; }
    /** @type {?} */
    var decorator = function (target, propertyKey) {
        Object.defineProperty(target, propertyKey, {
            set: /**
             * @this {?}
             * @param {?} value
             * @return {?}
             */
            function (value) {
                value = typeof value === 'string' ? Number.parseFloat(value) : value;
                /** @type {?} */
                var audioParam = this instanceof AudioWorkletNode
                    ? ((/** @type {?} */ (this.parameters))).get(propertyKey)
                    : this[(/** @type {?} */ (param))];
                if (audioParam instanceof AudioParam) {
                    processAudioParam(audioParam, value, this.context.currentTime);
                }
                else {
                    // Fallback for older browsers
                    Object.defineProperty(target, propertyKey, { value: value });
                }
            },
        });
    };
    return decorator;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/** @type {?} */
var AUDIO_CONTEXT = new InjectionToken('Web Audio API context', {
    providedIn: 'root',
    factory: function () { return new AudioContext(); },
});

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * @param {?} latencyHint
 * @return {?}
 */
function latencyHintFactory(latencyHint) {
    return latencyHint === null
        ? undefined
        : Number.parseFloat(latencyHint) || latencyHint;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
// @dynamic
var WebAudioContext = /** @class */ (function (_super) {
    __extends(WebAudioContext, _super);
    function WebAudioContext(latencyHint, sampleRate) {
        return _super.call(this, {
            latencyHint: latencyHintFactory(latencyHint),
            sampleRate: parseInt(sampleRate || '', 10) || undefined,
        }) || this;
    }
    /**
     * @return {?}
     */
    WebAudioContext.prototype.ngOnDestroy = /**
     * @return {?}
     */
    function () {
        this.close();
    };
    WebAudioContext.decorators = [
        { type: Directive, args: [{
                    selector: '[waAudioContext]',
                    providers: [
                        {
                            provide: AUDIO_CONTEXT,
                            useExisting: forwardRef(function () { return WebAudioContext; }),
                        },
                    ],
                },] }
    ];
    /** @nocollapse */
    WebAudioContext.ctorParameters = function () { return [
        { type: undefined, decorators: [{ type: Attribute, args: ['latencyHint',] }] },
        { type: undefined, decorators: [{ type: Attribute, args: ['sampleRate',] }] }
    ]; };
    return WebAudioContext;
}(AudioContext));

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * This is mostly for internal use only
 * @type {?}
 */
var CONSTRUCTOR_SUPPORT = new InjectionToken('Tests if constructor mode of node creation is supported or a fallback to factory method is needed', {
    providedIn: 'root',
    factory: function () {
        try {
            return !!new GainNode(inject(AUDIO_CONTEXT));
        }
        catch (_) {
            return false;
        }
    },
});

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
// @dynamic
var WebAudioChannel = /** @class */ (function (_super) {
    __extends(WebAudioChannel, _super);
    function WebAudioChannel(context, modern) {
        var _this = this;
        if (modern) {
            _this = _super.call(this, context) || this;
        }
        else {
            /** @type {?} */
            var result = context.createGain();
            Object.setPrototypeOf(result, WebAudioChannel.prototype);
            return (/** @type {?} */ (result));
        }
        return _this;
    }
    /**
     * @return {?}
     */
    WebAudioChannel.prototype.ngOnDestroy = /**
     * @return {?}
     */
    function () {
        this.disconnect();
    };
    WebAudioChannel.decorators = [
        { type: Directive, args: [{
                    selector: '[waChannel]',
                    exportAs: 'AudioNode',
                },] }
    ];
    /** @nocollapse */
    WebAudioChannel.ctorParameters = function () { return [
        { type: BaseAudioContext, decorators: [{ type: Inject, args: [AUDIO_CONTEXT,] }] },
        { type: Boolean, decorators: [{ type: Inject, args: [CONSTRUCTOR_SUPPORT,] }] }
    ]; };
    return WebAudioChannel;
}(GainNode));

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/** @type {?} */
var POLLING_TIME = 100;

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/** @type {?} */
var AUDIO_NODE = new InjectionToken('Web Audio API audio node', {
    factory: function () { return null; },
});

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * @param {?=} source
 * @param {?=} destination
 * @return {?}
 */
function connect(source, destination) {
    if (source && destination) {
        // @ts-ignore TS does not have a union override for connect method
        source.connect(destination);
    }
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
// @dynamic
var WebAudioDestination = /** @class */ (function (_super) {
    __extends(WebAudioDestination, _super);
    function WebAudioDestination(context, node, modern) {
        var _this = this;
        if (modern) {
            _this = _super.call(this, context) || this;
            WebAudioDestination.init(_this, node);
        }
        else {
            /** @type {?} */
            var result = (/** @type {?} */ (context.createAnalyser()));
            Object.setPrototypeOf(result, WebAudioDestination.prototype);
            WebAudioDestination.init(result, node);
            return result;
        }
        return _this;
    }
    /**
     * @return {?}
     */
    WebAudioDestination.prototype.ngOnDestroy = /**
     * @return {?}
     */
    function () {
        this.disconnect();
    };
    /**
     * @private
     * @param {?} array
     * @return {?}
     */
    WebAudioDestination.prototype.isSilent = /**
     * @private
     * @param {?} array
     * @return {?}
     */
    function (array) {
        return Math.abs(array.reduce(function (acc, cur) { return acc + cur; }, 0)) < 0.001;
    };
    /**
     * @param {?} that
     * @param {?} node
     * @return {?}
     */
    WebAudioDestination.init = /**
     * @param {?} that
     * @param {?} node
     * @return {?}
     */
    function (that, node) {
        connect(node, that);
        that.fftSize = 256;
        that.connect(that.context.destination);
        that.quiet = interval(POLLING_TIME).pipe(mapTo(new Float32Array(that.fftSize)), tap(function (array) { return that.getFloatTimeDomainData(array); }), map(function (array) { return that.isSilent(array); }), distinctUntilChanged(), skipWhile(function (isSilent) { return isSilent; }), debounceTime(5000), filter(function (isSilent) { return isSilent; }));
    };
    WebAudioDestination.decorators = [
        { type: Directive, args: [{
                    selector: '[waAudioDestinationNode]',
                    exportAs: 'AudioNode',
                },] }
    ];
    /** @nocollapse */
    WebAudioDestination.ctorParameters = function () { return [
        { type: BaseAudioContext, decorators: [{ type: Inject, args: [AUDIO_CONTEXT,] }] },
        { type: undefined, decorators: [{ type: Inject, args: [AUDIO_NODE,] }] },
        { type: Boolean, decorators: [{ type: Inject, args: [CONSTRUCTOR_SUPPORT,] }] }
    ]; };
    WebAudioDestination.propDecorators = {
        quiet: [{ type: Output }]
    };
    return WebAudioDestination;
}(AnalyserNode));

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * @param {?=} value
 * @return {?}
 */
function fallbackAudioParam(value) {
    if (!value) {
        return 0;
    }
    if (typeof value === 'number') {
        return value;
    }
    if (value instanceof Array) {
        /** @type {?} */
        var last = value[value.length - 1].value;
        return typeof last === 'number' ? last : last[last.length - 1];
    }
    if (value.value instanceof Array) {
        return value.value[value.value.length - 1];
    }
    return value.value;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
// @dynamic
var WebAudioListener = /** @class */ (function (_super) {
    __extends(WebAudioListener, _super);
    function WebAudioListener(context, modern) {
        var _this = this;
        if (modern) {
            _this = _super.call(this, context) || this;
        }
        else {
            /** @type {?} */
            var result = context.createGain();
            Object.setPrototypeOf(result, WebAudioListener.prototype);
            return (/** @type {?} */ (result));
        }
        return _this;
    }
    Object.defineProperty(WebAudioListener.prototype, "forwardX", {
        get: /**
         * @return {?}
         */
        function () {
            return this.context.listener.forwardX;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(WebAudioListener.prototype, "forwardY", {
        get: /**
         * @return {?}
         */
        function () {
            return this.context.listener.forwardY;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(WebAudioListener.prototype, "forwardZ", {
        get: /**
         * @return {?}
         */
        function () {
            return this.context.listener.forwardZ;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(WebAudioListener.prototype, "positionX", {
        get: /**
         * @return {?}
         */
        function () {
            return this.context.listener.positionX;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(WebAudioListener.prototype, "positionY", {
        get: /**
         * @return {?}
         */
        function () {
            return this.context.listener.positionY;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(WebAudioListener.prototype, "positionZ", {
        get: /**
         * @return {?}
         */
        function () {
            return this.context.listener.positionZ;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(WebAudioListener.prototype, "upX", {
        get: /**
         * @return {?}
         */
        function () {
            return this.context.listener.upX;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(WebAudioListener.prototype, "upY", {
        get: /**
         * @return {?}
         */
        function () {
            return this.context.listener.upY;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(WebAudioListener.prototype, "upZ", {
        get: /**
         * @return {?}
         */
        function () {
            return this.context.listener.upZ;
        },
        enumerable: true,
        configurable: true
    });
    /**
     * @return {?}
     */
    WebAudioListener.prototype.ngOnChanges = /**
     * @return {?}
     */
    function () {
        if (this.context.listener.positionX instanceof AudioParam) {
            return;
        }
        // Fallback for older browsers
        this.context.listener.setOrientation(fallbackAudioParam(this.forwardXParam), fallbackAudioParam(this.forwardYParam), fallbackAudioParam(this.forwardZParam), fallbackAudioParam(this.upXParam), fallbackAudioParam(this.upYParam), fallbackAudioParam(this.upZParam));
        this.context.listener.setPosition(fallbackAudioParam(this.positionXParam), fallbackAudioParam(this.positionYParam), fallbackAudioParam(this.positionZParam));
    };
    WebAudioListener.decorators = [
        { type: Directive, args: [{
                    selector: '[waAudioContext],[waOfflineAudioContext][length][sampleRate]',
                },] }
    ];
    /** @nocollapse */
    WebAudioListener.ctorParameters = function () { return [
        { type: BaseAudioContext, decorators: [{ type: Self }, { type: Inject, args: [AUDIO_CONTEXT,] }] },
        { type: Boolean, decorators: [{ type: Inject, args: [CONSTRUCTOR_SUPPORT,] }] }
    ]; };
    WebAudioListener.propDecorators = {
        forwardXParam: [{ type: Input, args: ['forwardX',] }],
        forwardYParam: [{ type: Input, args: ['forwardY',] }],
        forwardZParam: [{ type: Input, args: ['forwardZ',] }],
        positionXParam: [{ type: Input, args: ['positionX',] }],
        positionYParam: [{ type: Input, args: ['positionY',] }],
        positionZParam: [{ type: Input, args: ['positionZ',] }],
        upXParam: [{ type: Input, args: ['upX',] }],
        upYParam: [{ type: Input, args: ['upY',] }],
        upZParam: [{ type: Input, args: ['upZ',] }]
    };
    __decorate([
        audioParam('forwardX'),
        __metadata("design:type", Object)
    ], WebAudioListener.prototype, "forwardXParam", void 0);
    __decorate([
        audioParam('forwardY'),
        __metadata("design:type", Object)
    ], WebAudioListener.prototype, "forwardYParam", void 0);
    __decorate([
        audioParam('forwardZ'),
        __metadata("design:type", Object)
    ], WebAudioListener.prototype, "forwardZParam", void 0);
    __decorate([
        audioParam('positionX'),
        __metadata("design:type", Object)
    ], WebAudioListener.prototype, "positionXParam", void 0);
    __decorate([
        audioParam('positionY'),
        __metadata("design:type", Object)
    ], WebAudioListener.prototype, "positionYParam", void 0);
    __decorate([
        audioParam('positionZ'),
        __metadata("design:type", Object)
    ], WebAudioListener.prototype, "positionZParam", void 0);
    __decorate([
        audioParam('upX'),
        __metadata("design:type", Object)
    ], WebAudioListener.prototype, "upXParam", void 0);
    __decorate([
        audioParam('upY'),
        __metadata("design:type", Object)
    ], WebAudioListener.prototype, "upYParam", void 0);
    __decorate([
        audioParam('upZ'),
        __metadata("design:type", Object)
    ], WebAudioListener.prototype, "upZParam", void 0);
    return WebAudioListener;
}(GainNode));

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
// @dynamic
var WebAudioOfflineContext = /** @class */ (function (_super) {
    __extends(WebAudioOfflineContext, _super);
    function WebAudioOfflineContext(length, sampleRate, numberOfChannels, autoplay) {
        var _this = _super.call(this, parseInt(numberOfChannels || '', 10) || 1, parseInt(length, 10), parseInt(sampleRate, 10)) || this;
        _this.complete = new EventEmitter();
        _this.oncomplete = function (_a) {
            var renderedBuffer = _a.renderedBuffer;
            return _this.complete.emit(renderedBuffer);
        };
        if (autoplay !== null) {
            _this.startRendering();
        }
        return _this;
    }
    WebAudioOfflineContext.decorators = [
        { type: Directive, args: [{
                    selector: '[waOfflineAudioContext][length][sampleRate]',
                    providers: [
                        {
                            provide: AUDIO_CONTEXT,
                            useExisting: forwardRef(function () { return WebAudioOfflineContext; }),
                        },
                    ],
                },] }
    ];
    /** @nocollapse */
    WebAudioOfflineContext.ctorParameters = function () { return [
        { type: String, decorators: [{ type: Attribute, args: ['length',] }] },
        { type: String, decorators: [{ type: Attribute, args: ['sampleRate',] }] },
        { type: undefined, decorators: [{ type: Attribute, args: ['numberOfChannels',] }] },
        { type: undefined, decorators: [{ type: Attribute, args: ['autoplay',] }] }
    ]; };
    WebAudioOfflineContext.propDecorators = {
        complete: [{ type: Output }]
    };
    return WebAudioOfflineContext;
}(OfflineAudioContext));

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
// @dynamic
var WebAudioOutput = /** @class */ (function (_super) {
    __extends(WebAudioOutput, _super);
    function WebAudioOutput(context, node, modern) {
        var _this = this;
        if (modern) {
            _this = _super.call(this, context) || this;
            connect(node, _this);
        }
        else {
            /** @type {?} */
            var result = context.createGain();
            Object.setPrototypeOf(result, WebAudioOutput.prototype);
            connect(node, result);
            return (/** @type {?} */ (result));
        }
        return _this;
    }
    Object.defineProperty(WebAudioOutput.prototype, "waOutput", {
        set: /**
         * @param {?} destination
         * @return {?}
         */
        function (destination) {
            this.disconnect();
            connect(this, destination);
        },
        enumerable: true,
        configurable: true
    });
    /**
     * @return {?}
     */
    WebAudioOutput.prototype.ngOnDestroy = /**
     * @return {?}
     */
    function () {
        this.disconnect();
    };
    /**
     * @param {?} that
     * @param {?} node
     * @return {?}
     */
    WebAudioOutput.init = /**
     * @param {?} that
     * @param {?} node
     * @return {?}
     */
    function (that, node) {
        connect(node, that);
    };
    WebAudioOutput.decorators = [
        { type: Directive, args: [{
                    selector: '[waOutput]',
                },] }
    ];
    /** @nocollapse */
    WebAudioOutput.ctorParameters = function () { return [
        { type: BaseAudioContext, decorators: [{ type: Inject, args: [AUDIO_CONTEXT,] }] },
        { type: undefined, decorators: [{ type: Inject, args: [AUDIO_NODE,] }] },
        { type: Boolean, decorators: [{ type: Inject, args: [CONSTRUCTOR_SUPPORT,] }] }
    ]; };
    WebAudioOutput.propDecorators = {
        waOutput: [{ type: Input }]
    };
    return WebAudioOutput;
}(GainNode));

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
// @dynamic
var WebAudioMediaStreamDestination = /** @class */ (function (_super) {
    __extends(WebAudioMediaStreamDestination, _super);
    function WebAudioMediaStreamDestination(context, node, modern) {
        var _this = this;
        if (modern) {
            _this = _super.call(this, context) || this;
            connect(node, _this);
        }
        else {
            /** @type {?} */
            var result = context.createMediaStreamDestination();
            Object.setPrototypeOf(result, WebAudioMediaStreamDestination.prototype);
            connect(node, result);
            return (/** @type {?} */ (result));
        }
        return _this;
    }
    WebAudioMediaStreamDestination.decorators = [
        { type: Directive, args: [{
                    selector: '[waMediaStreamAudioDestinationNode]',
                    exportAs: 'AudioNode',
                },] }
    ];
    /** @nocollapse */
    WebAudioMediaStreamDestination.ctorParameters = function () { return [
        { type: AudioContext, decorators: [{ type: Inject, args: [AUDIO_CONTEXT,] }] },
        { type: undefined, decorators: [{ type: Inject, args: [AUDIO_NODE,] }] },
        { type: Boolean, decorators: [{ type: Inject, args: [CONSTRUCTOR_SUPPORT,] }] }
    ]; };
    return WebAudioMediaStreamDestination;
}(MediaStreamAudioDestinationNode));

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * @param {?} value
 * @param {?} fallback
 * @return {?}
 */
function parse(value, fallback) {
    /** @type {?} */
    var parsed = parseFloat(value || '');
    return isNaN(parsed) ? fallback : parsed;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
// @dynamic
var WebAudioAnalyser = /** @class */ (function (_super) {
    __extends(WebAudioAnalyser, _super);
    function WebAudioAnalyser(context, node, modern, fftSizeArg, maxDecibelsArg, minDecibelsArg, smoothingTimeConstantArg) {
        var _this = this;
        /** @type {?} */
        var fftSize = parse(fftSizeArg, 2048);
        /** @type {?} */
        var maxDecibels = parse(maxDecibelsArg, -30);
        /** @type {?} */
        var minDecibels = parse(minDecibelsArg, -100);
        /** @type {?} */
        var smoothingTimeConstant = parse(smoothingTimeConstantArg, 0.8);
        if (modern) {
            _this = _super.call(this, context, { fftSize: fftSize, maxDecibels: maxDecibels, minDecibels: minDecibels, smoothingTimeConstant: smoothingTimeConstant }) || this;
            WebAudioAnalyser.init(_this, node);
        }
        else {
            /** @type {?} */
            var result = (/** @type {?} */ (context.createAnalyser()));
            Object.setPrototypeOf(result, WebAudioAnalyser.prototype);
            WebAudioAnalyser.init(result, node);
            result.fftSize = fftSize;
            result.maxDecibels = maxDecibels;
            result.minDecibels = minDecibels;
            result.smoothingTimeConstant = smoothingTimeConstant;
            return result;
        }
        return _this;
    }
    /**
     * @return {?}
     */
    WebAudioAnalyser.prototype.ngOnDestroy = /**
     * @return {?}
     */
    function () {
        this.disconnect();
    };
    /**
     * @param {?} that
     * @param {?} node
     * @return {?}
     */
    WebAudioAnalyser.init = /**
     * @param {?} that
     * @param {?} node
     * @return {?}
     */
    function (that, node) {
        connect(node, that);
        that.frequencyByte$ = interval(0, animationFrameScheduler).pipe(mapTo(new Uint8Array(that.frequencyBinCount)), map(function (array) {
            return array.length === that.frequencyBinCount
                ? array
                : new Uint8Array(that.frequencyBinCount);
        }), tap(function (array) { return that.getByteFrequencyData(array); }));
        that.frequencyFloat$ = interval(0, animationFrameScheduler).pipe(mapTo(new Float32Array(that.frequencyBinCount)), map(function (array) {
            return array.length === that.frequencyBinCount
                ? array
                : new Float32Array(that.frequencyBinCount);
        }), tap(function (array) { return that.getFloatFrequencyData(array); }));
        that.timeByte$ = interval(0, animationFrameScheduler).pipe(mapTo(new Uint8Array(that.fftSize)), map(function (array) {
            return array.length === that.fftSize ? array : new Uint8Array(that.fftSize);
        }), tap(function (array) { return that.getByteTimeDomainData(array); }));
        that.timeFloat$ = interval(0, animationFrameScheduler).pipe(mapTo(new Float32Array(that.fftSize)), map(function (array) {
            return array.length === that.fftSize ? array : new Float32Array(that.fftSize);
        }), tap(function (array) { return that.getFloatTimeDomainData(array); }));
    };
    WebAudioAnalyser.decorators = [
        { type: Directive, args: [{
                    selector: '[waAnalyserNode]',
                    exportAs: 'AudioNode',
                    inputs: [
                        'fftSize',
                        'minDecibels',
                        'maxDecibels',
                        'smoothingTimeConstant',
                        'channelCount',
                        'channelCountMode',
                        'channelInterpretation',
                    ],
                    providers: [
                        {
                            provide: AUDIO_NODE,
                            useExisting: forwardRef(function () { return WebAudioAnalyser; }),
                        },
                    ],
                },] }
    ];
    /** @nocollapse */
    WebAudioAnalyser.ctorParameters = function () { return [
        { type: BaseAudioContext, decorators: [{ type: Inject, args: [AUDIO_CONTEXT,] }] },
        { type: undefined, decorators: [{ type: SkipSelf }, { type: Inject, args: [AUDIO_NODE,] }] },
        { type: Boolean, decorators: [{ type: Inject, args: [CONSTRUCTOR_SUPPORT,] }] },
        { type: undefined, decorators: [{ type: Attribute, args: ['fftSize',] }] },
        { type: undefined, decorators: [{ type: Attribute, args: ['maxDecibels',] }] },
        { type: undefined, decorators: [{ type: Attribute, args: ['minDecibels',] }] },
        { type: undefined, decorators: [{ type: Attribute, args: ['smoothingTimeConstant',] }] }
    ]; };
    WebAudioAnalyser.propDecorators = {
        frequencyByte$: [{ type: Output }],
        frequencyFloat$: [{ type: Output }],
        timeByte$: [{ type: Output }],
        timeFloat$: [{ type: Output }]
    };
    return WebAudioAnalyser;
}(AnalyserNode));

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
// @dynamic
var WebAudioBiquadFilter = /** @class */ (function (_super) {
    __extends(WebAudioBiquadFilter, _super);
    function WebAudioBiquadFilter(context, modern, node, detuneArg, frequencyArg, gainArg, QArg) {
        var _this = this;
        /** @type {?} */
        var detune = parse(detuneArg, 0);
        /** @type {?} */
        var frequency = parse(frequencyArg, 350);
        /** @type {?} */
        var gain = parse(gainArg, 0);
        /** @type {?} */
        var Q = parse(QArg, 1);
        if (modern) {
            _this = _super.call(this, context, { detune: detune, frequency: frequency, gain: gain, Q: Q }) || this;
            connect(node, _this);
        }
        else {
            /** @type {?} */
            var result = (/** @type {?} */ (context.createBiquadFilter()));
            Object.setPrototypeOf(result, WebAudioBiquadFilter.prototype);
            result.detune.value = detune;
            result.frequency.value = frequency;
            result.gain.value = gain;
            result.Q.value = Q;
            connect(node, result);
            return result;
        }
        return _this;
    }
    /**
     * @return {?}
     */
    WebAudioBiquadFilter.prototype.ngOnDestroy = /**
     * @return {?}
     */
    function () {
        this.disconnect();
    };
    /**
     * @param {?} that
     * @param {?} node
     * @return {?}
     */
    WebAudioBiquadFilter.init = /**
     * @param {?} that
     * @param {?} node
     * @return {?}
     */
    function (that, node) {
        connect(node, that);
    };
    WebAudioBiquadFilter.decorators = [
        { type: Directive, args: [{
                    selector: '[waBiquadFilterNode]',
                    exportAs: 'AudioNode',
                    inputs: ['type', 'channelCount', 'channelCountMode', 'channelInterpretation'],
                    providers: [
                        {
                            provide: AUDIO_NODE,
                            useExisting: forwardRef(function () { return WebAudioBiquadFilter; }),
                        },
                    ],
                },] }
    ];
    /** @nocollapse */
    WebAudioBiquadFilter.ctorParameters = function () { return [
        { type: BaseAudioContext, decorators: [{ type: Inject, args: [AUDIO_CONTEXT,] }] },
        { type: Boolean, decorators: [{ type: Inject, args: [CONSTRUCTOR_SUPPORT,] }] },
        { type: undefined, decorators: [{ type: SkipSelf }, { type: Inject, args: [AUDIO_NODE,] }] },
        { type: undefined, decorators: [{ type: Attribute, args: ['detune',] }] },
        { type: undefined, decorators: [{ type: Attribute, args: ['frequency',] }] },
        { type: undefined, decorators: [{ type: Attribute, args: ['gain',] }] },
        { type: undefined, decorators: [{ type: Attribute, args: ['Q',] }] }
    ]; };
    WebAudioBiquadFilter.propDecorators = {
        detuneParam: [{ type: Input, args: ['detune',] }],
        frequencyParam: [{ type: Input, args: ['frequency',] }],
        gainParam: [{ type: Input, args: ['gain',] }],
        qParam: [{ type: Input, args: ['Q',] }]
    };
    __decorate([
        audioParam('detune'),
        __metadata("design:type", Object)
    ], WebAudioBiquadFilter.prototype, "detuneParam", void 0);
    __decorate([
        audioParam('frequency'),
        __metadata("design:type", Object)
    ], WebAudioBiquadFilter.prototype, "frequencyParam", void 0);
    __decorate([
        audioParam('gain'),
        __metadata("design:type", Object)
    ], WebAudioBiquadFilter.prototype, "gainParam", void 0);
    __decorate([
        audioParam('Q'),
        __metadata("design:type", Object)
    ], WebAudioBiquadFilter.prototype, "qParam", void 0);
    return WebAudioBiquadFilter;
}(BiquadFilterNode));

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
// @dynamic
var WebAudioChannelMerger = /** @class */ (function (_super) {
    __extends(WebAudioChannelMerger, _super);
    function WebAudioChannelMerger(inputs, context, modern) {
        var _this = this;
        /** @type {?} */
        var numberOfInputs = parseInt(inputs || '', 10) || 6;
        if (modern) {
            _this = _super.call(this, context, { numberOfInputs: numberOfInputs }) || this;
        }
        else {
            /** @type {?} */
            var result = context.createChannelMerger(numberOfInputs);
            Object.setPrototypeOf(result, WebAudioChannelMerger.prototype);
            return (/** @type {?} */ (result));
        }
        return _this;
    }
    Object.defineProperty(WebAudioChannelMerger.prototype, "channels", {
        set: /**
         * @param {?} channels
         * @return {?}
         */
        function (channels) {
            var _this = this;
            channels.forEach(function (node, index) {
                node.connect(_this, 0, index);
            });
        },
        enumerable: true,
        configurable: true
    });
    /**
     * @return {?}
     */
    WebAudioChannelMerger.prototype.ngOnDestroy = /**
     * @return {?}
     */
    function () {
        this.disconnect();
    };
    WebAudioChannelMerger.decorators = [
        { type: Directive, args: [{
                    selector: '[waChannelMergerNode]',
                    exportAs: 'AudioNode',
                    inputs: ['channelCount', 'channelCountMode', 'channelInterpretation'],
                    providers: [
                        {
                            provide: AUDIO_NODE,
                            useExisting: forwardRef(function () { return WebAudioChannelMerger; }),
                        },
                    ],
                },] }
    ];
    /** @nocollapse */
    WebAudioChannelMerger.ctorParameters = function () { return [
        { type: undefined, decorators: [{ type: Attribute, args: ['numberOfInputs',] }] },
        { type: BaseAudioContext, decorators: [{ type: Inject, args: [AUDIO_CONTEXT,] }] },
        { type: Boolean, decorators: [{ type: Inject, args: [CONSTRUCTOR_SUPPORT,] }] }
    ]; };
    WebAudioChannelMerger.propDecorators = {
        channels: [{ type: ContentChildren, args: [WebAudioChannel, { descendants: false },] }]
    };
    return WebAudioChannelMerger;
}(ChannelMergerNode));

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
// @dynamic
var WebAudioChannelSplitter = /** @class */ (function (_super) {
    __extends(WebAudioChannelSplitter, _super);
    function WebAudioChannelSplitter(outputs, context, node, modern) {
        var _this = this;
        /** @type {?} */
        var numberOfOutputs = parseInt(outputs || '', 10) || 6;
        if (modern) {
            _this = _super.call(this, context, { numberOfOutputs: numberOfOutputs }) || this;
            connect(node, _this);
        }
        else {
            /** @type {?} */
            var result = context.createChannelSplitter(numberOfOutputs);
            Object.setPrototypeOf(result, WebAudioChannelSplitter.prototype);
            connect(node, result);
            return (/** @type {?} */ (result));
        }
        return _this;
    }
    Object.defineProperty(WebAudioChannelSplitter.prototype, "channels", {
        set: /**
         * @param {?} channels
         * @return {?}
         */
        function (channels) {
            var _this = this;
            this.disconnect();
            channels
                .filter(function (node) { return !!node; })
                .forEach(function (node, index) {
                _this.connect((/** @type {?} */ (node)), index);
            });
        },
        enumerable: true,
        configurable: true
    });
    /**
     * @return {?}
     */
    WebAudioChannelSplitter.prototype.ngOnDestroy = /**
     * @return {?}
     */
    function () {
        this.disconnect();
    };
    WebAudioChannelSplitter.decorators = [
        { type: Directive, args: [{
                    selector: '[waChannelSplitterNode]',
                    exportAs: 'AudioNode',
                    inputs: ['channelCount', 'channelCountMode', 'channelInterpretation'],
                    providers: [
                        {
                            provide: AUDIO_NODE,
                            useValue: null,
                        },
                    ],
                },] }
    ];
    /** @nocollapse */
    WebAudioChannelSplitter.ctorParameters = function () { return [
        { type: undefined, decorators: [{ type: Attribute, args: ['numberOfOutputs',] }] },
        { type: BaseAudioContext, decorators: [{ type: Inject, args: [AUDIO_CONTEXT,] }] },
        { type: undefined, decorators: [{ type: SkipSelf }, { type: Inject, args: [AUDIO_NODE,] }] },
        { type: Boolean, decorators: [{ type: Inject, args: [CONSTRUCTOR_SUPPORT,] }] }
    ]; };
    WebAudioChannelSplitter.propDecorators = {
        channels: [{ type: ContentChildren, args: [(/** @type {?} */ (AUDIO_NODE)), { descendants: false },] }]
    };
    return WebAudioChannelSplitter;
}(ChannelSplitterNode));

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
// @dynamic
var AudioBufferService = /** @class */ (function () {
    function AudioBufferService(context) {
        this.context = context;
        this.cache = new Map();
    }
    /**
     * @param {?} url
     * @return {?}
     */
    AudioBufferService.prototype.fetch = /**
     * @param {?} url
     * @return {?}
     */
    function (url) {
        var _this = this;
        return new Promise(function (resolve, reject) {
            if (_this.cache.has(url)) {
                resolve(_this.cache.get(url));
                return;
            }
            /** @type {?} */
            var request = new XMLHttpRequest();
            request.open('GET', url, true);
            request.responseType = 'arraybuffer';
            request.onerror = reject;
            request.onabort = reject;
            request.onload = function () {
                _this.context.decodeAudioData(request.response, function (buffer) {
                    _this.cache.set(url, buffer);
                    resolve(buffer);
                }, reject);
            };
            request.send();
        });
    };
    AudioBufferService.decorators = [
        { type: Injectable, args: [{
                    providedIn: 'root',
                },] }
    ];
    /** @nocollapse */
    AudioBufferService.ctorParameters = function () { return [
        { type: BaseAudioContext, decorators: [{ type: Inject, args: [AUDIO_CONTEXT,] }] }
    ]; };
    /** @nocollapse */ AudioBufferService.ngInjectableDef = defineInjectable({ factory: function AudioBufferService_Factory() { return new AudioBufferService(inject(AUDIO_CONTEXT)); }, token: AudioBufferService, providedIn: "root" });
    return AudioBufferService;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
// @dynamic
var WebAudioConvolver = /** @class */ (function (_super) {
    __extends(WebAudioConvolver, _super);
    function WebAudioConvolver(audioBufferService, context, node, modern) {
        var _this = this;
        if (modern) {
            _this = _super.call(this, context) || this;
            WebAudioConvolver.init(_this, node, audioBufferService);
        }
        else {
            /** @type {?} */
            var result = (/** @type {?} */ (context.createConvolver()));
            Object.setPrototypeOf(result, WebAudioConvolver.prototype);
            WebAudioConvolver.init(result, node, audioBufferService);
            return result;
        }
        return _this;
    }
    Object.defineProperty(WebAudioConvolver.prototype, "bufferSetter", {
        set: /**
         * @param {?} source
         * @return {?}
         */
        function (source) {
            this.buffer$.next(source);
        },
        enumerable: true,
        configurable: true
    });
    /**
     * @return {?}
     */
    WebAudioConvolver.prototype.ngOnDestroy = /**
     * @return {?}
     */
    function () {
        this.buffer$.complete();
        this.disconnect();
    };
    /**
     * @param {?} that
     * @param {?} node
     * @param {?} audioBufferService
     * @return {?}
     */
    WebAudioConvolver.init = /**
     * @param {?} that
     * @param {?} node
     * @param {?} audioBufferService
     * @return {?}
     */
    function (that, node, audioBufferService) {
        connect(node, that);
        that.buffer$ = new Subject();
        that.buffer$
            .pipe(switchMap(function (source) {
            return typeof source === 'string'
                ? audioBufferService.fetch(source)
                : of(source);
        }))
            .subscribe(function (buffer) {
            that.buffer = buffer;
        });
    };
    WebAudioConvolver.decorators = [
        { type: Directive, args: [{
                    selector: '[waConvolverNode]',
                    exportAs: 'AudioNode',
                    inputs: ['normalize', 'channelCount', 'channelCountMode', 'channelInterpretation'],
                    providers: [
                        {
                            provide: AUDIO_NODE,
                            useExisting: forwardRef(function () { return WebAudioConvolver; }),
                        },
                    ],
                },] }
    ];
    /** @nocollapse */
    WebAudioConvolver.ctorParameters = function () { return [
        { type: AudioBufferService, decorators: [{ type: Inject, args: [AudioBufferService,] }] },
        { type: BaseAudioContext, decorators: [{ type: Inject, args: [AUDIO_CONTEXT,] }] },
        { type: undefined, decorators: [{ type: SkipSelf }, { type: Inject, args: [AUDIO_NODE,] }] },
        { type: Boolean, decorators: [{ type: Inject, args: [CONSTRUCTOR_SUPPORT,] }] }
    ]; };
    WebAudioConvolver.propDecorators = {
        bufferSetter: [{ type: Input, args: ['buffer',] }]
    };
    return WebAudioConvolver;
}(ConvolverNode));

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
// @dynamic
var WebAudioDelay = /** @class */ (function (_super) {
    __extends(WebAudioDelay, _super);
    function WebAudioDelay(context, node, modern, delayTimeArg, maxDelayTimeArg) {
        var _this = this;
        /** @type {?} */
        var delayTime = parse(delayTimeArg, 0);
        /** @type {?} */
        var maxDelayTime = parse(maxDelayTimeArg, 1);
        if (modern) {
            _this = _super.call(this, context, { delayTime: delayTime, maxDelayTime: maxDelayTime }) || this;
            connect(node, _this);
        }
        else {
            /** @type {?} */
            var result = (/** @type {?} */ (context.createDelay(maxDelayTime)));
            Object.setPrototypeOf(result, WebAudioDelay.prototype);
            connect(node, result);
            result.delayTime.value = delayTime;
            return result;
        }
        return _this;
    }
    /**
     * @return {?}
     */
    WebAudioDelay.prototype.ngOnDestroy = /**
     * @return {?}
     */
    function () {
        this.disconnect();
    };
    /**
     * @param {?} that
     * @param {?} node
     * @return {?}
     */
    WebAudioDelay.init = /**
     * @param {?} that
     * @param {?} node
     * @return {?}
     */
    function (that, node) {
        connect(node, that);
    };
    WebAudioDelay.decorators = [
        { type: Directive, args: [{
                    selector: '[waDelayNode]',
                    exportAs: 'AudioNode',
                    inputs: ['channelCount', 'channelCountMode', 'channelInterpretation'],
                    providers: [
                        {
                            provide: AUDIO_NODE,
                            useExisting: forwardRef(function () { return WebAudioDelay; }),
                        },
                    ],
                },] }
    ];
    /** @nocollapse */
    WebAudioDelay.ctorParameters = function () { return [
        { type: BaseAudioContext, decorators: [{ type: Inject, args: [AUDIO_CONTEXT,] }] },
        { type: undefined, decorators: [{ type: SkipSelf }, { type: Inject, args: [AUDIO_NODE,] }] },
        { type: Boolean, decorators: [{ type: Inject, args: [CONSTRUCTOR_SUPPORT,] }] },
        { type: undefined, decorators: [{ type: Attribute, args: ['delayTime',] }] },
        { type: undefined, decorators: [{ type: Attribute, args: ['maxDelayTime',] }] }
    ]; };
    WebAudioDelay.propDecorators = {
        delayTimeParam: [{ type: Input, args: ['delayTime',] }]
    };
    __decorate([
        audioParam('delayTime'),
        __metadata("design:type", Object)
    ], WebAudioDelay.prototype, "delayTimeParam", void 0);
    return WebAudioDelay;
}(DelayNode));

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
// @dynamic
var WebAudioDynamicsCompressor = /** @class */ (function (_super) {
    __extends(WebAudioDynamicsCompressor, _super);
    function WebAudioDynamicsCompressor(context, node, modern, attackArg, kneeArg, ratioArg, releaseArg, thresholdArg) {
        var _this = this;
        /** @type {?} */
        var attack = parse(attackArg, 0.003);
        /** @type {?} */
        var knee = parse(kneeArg, 30);
        /** @type {?} */
        var ratio = parse(ratioArg, 12);
        /** @type {?} */
        var release = parse(releaseArg, 0.25);
        /** @type {?} */
        var threshold = parse(thresholdArg, -24);
        if (modern) {
            _this = _super.call(this, context, { attack: attack, knee: knee, ratio: ratio, release: release, threshold: threshold }) || this;
            connect(node, _this);
        }
        else {
            /** @type {?} */
            var result = (/** @type {?} */ (context.createDynamicsCompressor()));
            Object.setPrototypeOf(result, WebAudioDynamicsCompressor.prototype);
            connect(node, result);
            result.attack.value = attack;
            result.knee.value = knee;
            result.ratio.value = ratio;
            result.release.value = release;
            result.threshold.value = threshold;
            return result;
        }
        return _this;
    }
    /**
     * @return {?}
     */
    WebAudioDynamicsCompressor.prototype.ngOnDestroy = /**
     * @return {?}
     */
    function () {
        this.disconnect();
    };
    WebAudioDynamicsCompressor.decorators = [
        { type: Directive, args: [{
                    selector: '[waDynamicsCompressorNode]',
                    exportAs: 'AudioNode',
                    inputs: ['channelCount', 'channelCountMode', 'channelInterpretation'],
                    providers: [
                        {
                            provide: AUDIO_NODE,
                            useExisting: forwardRef(function () { return WebAudioDynamicsCompressor; }),
                        },
                    ],
                },] }
    ];
    /** @nocollapse */
    WebAudioDynamicsCompressor.ctorParameters = function () { return [
        { type: BaseAudioContext, decorators: [{ type: Inject, args: [AUDIO_CONTEXT,] }] },
        { type: undefined, decorators: [{ type: SkipSelf }, { type: Inject, args: [AUDIO_NODE,] }] },
        { type: Boolean, decorators: [{ type: Inject, args: [CONSTRUCTOR_SUPPORT,] }] },
        { type: undefined, decorators: [{ type: Attribute, args: ['attack',] }] },
        { type: undefined, decorators: [{ type: Attribute, args: ['knee',] }] },
        { type: undefined, decorators: [{ type: Attribute, args: ['ratio',] }] },
        { type: undefined, decorators: [{ type: Attribute, args: ['release',] }] },
        { type: undefined, decorators: [{ type: Attribute, args: ['threshold',] }] }
    ]; };
    WebAudioDynamicsCompressor.propDecorators = {
        attackParam: [{ type: Input, args: ['attack',] }],
        kneeParam: [{ type: Input, args: ['knee',] }],
        ratioParam: [{ type: Input, args: ['ratio',] }],
        releaseParam: [{ type: Input, args: ['release',] }],
        thresholdParam: [{ type: Input, args: ['threshold',] }]
    };
    __decorate([
        audioParam('attack'),
        __metadata("design:type", Object)
    ], WebAudioDynamicsCompressor.prototype, "attackParam", void 0);
    __decorate([
        audioParam('knee'),
        __metadata("design:type", Object)
    ], WebAudioDynamicsCompressor.prototype, "kneeParam", void 0);
    __decorate([
        audioParam('ratio'),
        __metadata("design:type", Object)
    ], WebAudioDynamicsCompressor.prototype, "ratioParam", void 0);
    __decorate([
        audioParam('release'),
        __metadata("design:type", Object)
    ], WebAudioDynamicsCompressor.prototype, "releaseParam", void 0);
    __decorate([
        audioParam('threshold'),
        __metadata("design:type", Object)
    ], WebAudioDynamicsCompressor.prototype, "thresholdParam", void 0);
    return WebAudioDynamicsCompressor;
}(DynamicsCompressorNode));

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
// @dynamic
var WebAudioGain = /** @class */ (function (_super) {
    __extends(WebAudioGain, _super);
    function WebAudioGain(context, node, modern, gainArg) {
        var _this = this;
        /** @type {?} */
        var gain = parse(gainArg, 1);
        if (modern) {
            _this = _super.call(this, context, { gain: gain }) || this;
            connect(node, _this);
        }
        else {
            /** @type {?} */
            var result = (/** @type {?} */ (context.createGain()));
            Object.setPrototypeOf(result, WebAudioGain.prototype);
            connect(node, result);
            result.gain.value = gain;
            return result;
        }
        return _this;
    }
    /**
     * @return {?}
     */
    WebAudioGain.prototype.ngOnDestroy = /**
     * @return {?}
     */
    function () {
        this.disconnect();
    };
    WebAudioGain.decorators = [
        { type: Directive, args: [{
                    selector: '[waGainNode]',
                    exportAs: 'AudioNode',
                    inputs: ['channelCount', 'channelCountMode', 'channelInterpretation'],
                    providers: [
                        {
                            provide: AUDIO_NODE,
                            useExisting: forwardRef(function () { return WebAudioGain; }),
                        },
                    ],
                },] }
    ];
    /** @nocollapse */
    WebAudioGain.ctorParameters = function () { return [
        { type: BaseAudioContext, decorators: [{ type: Inject, args: [AUDIO_CONTEXT,] }] },
        { type: undefined, decorators: [{ type: SkipSelf }, { type: Inject, args: [AUDIO_NODE,] }] },
        { type: Boolean, decorators: [{ type: Inject, args: [CONSTRUCTOR_SUPPORT,] }] },
        { type: undefined, decorators: [{ type: Attribute, args: ['gain',] }] }
    ]; };
    WebAudioGain.propDecorators = {
        gainParam: [{ type: Input, args: ['gain',] }]
    };
    __decorate([
        audioParam('gain'),
        __metadata("design:type", Object)
    ], WebAudioGain.prototype, "gainParam", void 0);
    return WebAudioGain;
}(GainNode));

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/** @type {?} */
var FEEDBACK_COEFFICIENTS = new InjectionToken('A sequence of feedback coefficients for IIRFilterNode');

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/** @type {?} */
var FEEDFORWARD_COEFFICIENTS = new InjectionToken('A sequence of feedforward coefficients for IIRFilterNode');

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
// @dynamic
var WebAudioIIRFilter = /** @class */ (function (_super) {
    __extends(WebAudioIIRFilter, _super);
    function WebAudioIIRFilter(feedback, feedforward, context, modern, node) {
        var _this = this;
        if (modern) {
            _this = _super.call(this, context, { feedback: feedback, feedforward: feedforward }) || this;
            connect(node, _this);
        }
        else {
            /** @type {?} */
            var result = (/** @type {?} */ (context.createIIRFilter(feedback, feedforward)));
            Object.setPrototypeOf(result, WebAudioIIRFilter.prototype);
            connect(node, result);
            return result;
        }
        return _this;
    }
    /**
     * @return {?}
     */
    WebAudioIIRFilter.prototype.ngOnDestroy = /**
     * @return {?}
     */
    function () {
        this.disconnect();
    };
    WebAudioIIRFilter.decorators = [
        { type: Directive, args: [{
                    selector: '[waIIRFilterNode]',
                    exportAs: 'AudioNode',
                    inputs: ['channelCount', 'channelCountMode', 'channelInterpretation'],
                    providers: [
                        {
                            provide: AUDIO_NODE,
                            useExisting: forwardRef(function () { return WebAudioIIRFilter; }),
                        },
                    ],
                },] }
    ];
    /** @nocollapse */
    WebAudioIIRFilter.ctorParameters = function () { return [
        { type: Array, decorators: [{ type: Inject, args: [FEEDBACK_COEFFICIENTS,] }] },
        { type: Array, decorators: [{ type: Inject, args: [FEEDFORWARD_COEFFICIENTS,] }] },
        { type: BaseAudioContext, decorators: [{ type: Inject, args: [AUDIO_CONTEXT,] }] },
        { type: Boolean, decorators: [{ type: Inject, args: [CONSTRUCTOR_SUPPORT,] }] },
        { type: undefined, decorators: [{ type: SkipSelf }, { type: Inject, args: [AUDIO_NODE,] }] }
    ]; };
    return WebAudioIIRFilter;
}(IIRFilterNode));

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
// @dynamic
var WebAudioPanner = /** @class */ (function (_super) {
    __extends(WebAudioPanner, _super);
    function WebAudioPanner(context, node, modern) {
        var _this = this;
        if (modern) {
            _this = _super.call(this, context) || this;
            connect(node, _this);
        }
        else {
            /** @type {?} */
            var result = (/** @type {?} */ (context.createPanner()));
            Object.setPrototypeOf(result, WebAudioPanner.prototype);
            connect(node, result);
            return result;
        }
        return _this;
    }
    /**
     * @return {?}
     */
    WebAudioPanner.prototype.ngOnChanges = /**
     * @return {?}
     */
    function () {
        if (this.positionX instanceof AudioParam) {
            return;
        }
        // Fallback for older browsers
        this.setOrientation(fallbackAudioParam(this.orientationXParam), fallbackAudioParam(this.orientationYParam), fallbackAudioParam(this.orientationZParam));
        this.setPosition(fallbackAudioParam(this.positionXParam), fallbackAudioParam(this.positionYParam), fallbackAudioParam(this.positionZParam));
    };
    /**
     * @return {?}
     */
    WebAudioPanner.prototype.ngOnDestroy = /**
     * @return {?}
     */
    function () {
        this.disconnect();
    };
    WebAudioPanner.decorators = [
        { type: Directive, args: [{
                    selector: '[waPannerNode]',
                    exportAs: 'AudioNode',
                    inputs: [
                        'coneInnerAngle',
                        'coneOuterAngle',
                        'coneOuterGain',
                        'distanceModel',
                        'maxDistance',
                        'panningModel',
                        'refDistance',
                        'rolloffFactor',
                        'channelCount',
                        'channelCountMode',
                        'channelInterpretation',
                    ],
                    providers: [
                        {
                            provide: AUDIO_NODE,
                            useExisting: forwardRef(function () { return WebAudioPanner; }),
                        },
                    ],
                },] }
    ];
    /** @nocollapse */
    WebAudioPanner.ctorParameters = function () { return [
        { type: BaseAudioContext, decorators: [{ type: Inject, args: [AUDIO_CONTEXT,] }] },
        { type: undefined, decorators: [{ type: SkipSelf }, { type: Inject, args: [AUDIO_NODE,] }] },
        { type: Boolean, decorators: [{ type: Inject, args: [CONSTRUCTOR_SUPPORT,] }] }
    ]; };
    WebAudioPanner.propDecorators = {
        orientationXParam: [{ type: Input }],
        orientationYParam: [{ type: Input }],
        orientationZParam: [{ type: Input }],
        positionXParam: [{ type: Input }],
        positionYParam: [{ type: Input }],
        positionZParam: [{ type: Input }]
    };
    __decorate([
        audioParam('orientationX'),
        __metadata("design:type", Object)
    ], WebAudioPanner.prototype, "orientationXParam", void 0);
    __decorate([
        audioParam('orientationY'),
        __metadata("design:type", Object)
    ], WebAudioPanner.prototype, "orientationYParam", void 0);
    __decorate([
        audioParam('orientationZ'),
        __metadata("design:type", Object)
    ], WebAudioPanner.prototype, "orientationZParam", void 0);
    __decorate([
        audioParam('positionX'),
        __metadata("design:type", Object)
    ], WebAudioPanner.prototype, "positionXParam", void 0);
    __decorate([
        audioParam('positionY'),
        __metadata("design:type", Object)
    ], WebAudioPanner.prototype, "positionYParam", void 0);
    __decorate([
        audioParam('positionZ'),
        __metadata("design:type", Object)
    ], WebAudioPanner.prototype, "positionZParam", void 0);
    return WebAudioPanner;
}(PannerNode));

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
// @dynamic
var WebAudioScriptProcessor = /** @class */ (function (_super) {
    __extends(WebAudioScriptProcessor, _super);
    function WebAudioScriptProcessor(bufferSize, numberOfInputChannels, numberOfOutputChannels, context, node) {
        var _this = this;
        try {
            /** @type {?} */
            var result = (/** @type {?} */ (context.createScriptProcessor(parseInt(bufferSize || '', 10) || 0, parseInt(numberOfInputChannels || '', 10) || 2, parseInt(numberOfOutputChannels || '', 10) || 2)));
            Object.setPrototypeOf(result, WebAudioScriptProcessor.prototype);
            connect(node, result);
            /** @type {?} */
            var audioprocess_1 = new EventEmitter();
            result.audioprocess = audioprocess_1;
            result.onaudioprocess = function (e) { return audioprocess_1.emit(e); };
            return result;
        }
        catch (_) { }
        _this = _super.call(this) || this;
        return _this;
    }
    /**
     * @return {?}
     */
    WebAudioScriptProcessor.prototype.ngOnDestroy = /**
     * @return {?}
     */
    function () {
        this.disconnect();
    };
    WebAudioScriptProcessor.decorators = [
        { type: Directive, args: [{
                    selector: '[waScriptProcessorNode]',
                    exportAs: 'AudioNode',
                    inputs: ['channelCount', 'channelCountMode', 'channelInterpretation'],
                    providers: [
                        {
                            provide: AUDIO_NODE,
                            useExisting: forwardRef(function () { return WebAudioScriptProcessor; }),
                        },
                    ],
                },] }
    ];
    /** @nocollapse */
    WebAudioScriptProcessor.ctorParameters = function () { return [
        { type: undefined, decorators: [{ type: Attribute, args: ['bufferSize',] }] },
        { type: undefined, decorators: [{ type: Attribute, args: ['numberOfInputChannels',] }] },
        { type: undefined, decorators: [{ type: Attribute, args: ['numberOfOutputChannels',] }] },
        { type: BaseAudioContext, decorators: [{ type: Inject, args: [AUDIO_CONTEXT,] }] },
        { type: undefined, decorators: [{ type: SkipSelf }, { type: Inject, args: [AUDIO_NODE,] }] }
    ]; };
    WebAudioScriptProcessor.propDecorators = {
        audioprocess: [{ type: Output }]
    };
    return WebAudioScriptProcessor;
}(ScriptProcessorNode));

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
// @dynamic
var WebAudioStereoPanner = /** @class */ (function (_super) {
    __extends(WebAudioStereoPanner, _super);
    function WebAudioStereoPanner(context, node, panArg) {
        var _this = this;
        /** @type {?} */
        var pan = parse(panArg, 0);
        try {
            // @ts-ignore
            /** @type {?} */
            var _test = new StereoPannerNode(context);
        }
        catch (_) {
            /** @type {?} */
            var result = (/** @type {?} */ (((/** @type {?} */ (context.createPanner())))));
            Object.setPrototypeOf(result, WebAudioStereoPanner.prototype);
            result.fallbackToPannerNode(fallbackAudioParam(pan));
            connect(node, result);
            return result;
        }
        _this = _super.call(this, context, { pan: pan }) || this;
        connect(node, _this);
        return _this;
    }
    Object.defineProperty(WebAudioStereoPanner.prototype, "panParam", {
        set: /**
         * @param {?} pan
         * @return {?}
         */
        function (pan) {
            if ('setPosition' in this) {
                /** fallback for browsers not supporting {@link StereoPannerNode} */
                // @ts-ignore
                this.fallbackToPannerNode(fallbackAudioParam(pan));
            }
            else {
                processAudioParam(this.pan, pan, this.context.currentTime);
            }
        },
        enumerable: true,
        configurable: true
    });
    /**
     * @return {?}
     */
    WebAudioStereoPanner.prototype.ngOnDestroy = /**
     * @return {?}
     */
    function () {
        this.disconnect();
    };
    // @ts-ignore
    // @ts-ignore
    /**
     * @private
     * @param {?} pan
     * @return {?}
     */
    WebAudioStereoPanner.prototype.fallbackToPannerNode = 
    // @ts-ignore
    /**
     * @private
     * @param {?} pan
     * @return {?}
     */
    function (pan) {
        /** @type {?} */
        var xDeg = pan * 100;
        /** @type {?} */
        var zDeg = xDeg > 0 ? 270 - xDeg : xDeg + 90;
        /** @type {?} */
        var x = Math.sin(xDeg * (Math.PI / 180));
        /** @type {?} */
        var z = Math.sin(zDeg * (Math.PI / 180));
        // @ts-ignore
        this.setPosition(x, 0, z);
    };
    WebAudioStereoPanner.decorators = [
        { type: Directive, args: [{
                    selector: '[waStereoPannerNode]',
                    exportAs: 'AudioNode',
                    inputs: ['channelCount', 'channelCountMode', 'channelInterpretation'],
                    providers: [
                        {
                            provide: AUDIO_NODE,
                            useExisting: forwardRef(function () { return WebAudioStereoPanner; }),
                        },
                    ],
                },] }
    ];
    /** @nocollapse */
    WebAudioStereoPanner.ctorParameters = function () { return [
        { type: BaseAudioContext, decorators: [{ type: Inject, args: [AUDIO_CONTEXT,] }] },
        { type: undefined, decorators: [{ type: SkipSelf }, { type: Inject, args: [AUDIO_NODE,] }] },
        { type: undefined, decorators: [{ type: Attribute, args: ['pan',] }] }
    ]; };
    WebAudioStereoPanner.propDecorators = {
        panParam: [{ type: Input, args: ['pan',] }]
    };
    return WebAudioStereoPanner;
}(StereoPannerNode));

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
// @dynamic
var WebAudioWaveShaper = /** @class */ (function (_super) {
    __extends(WebAudioWaveShaper, _super);
    function WebAudioWaveShaper(context, node, modern) {
        var _this = this;
        if (modern) {
            _this = _super.call(this, context) || this;
            connect(node, _this);
        }
        else {
            /** @type {?} */
            var result = (/** @type {?} */ (context.createWaveShaper()));
            Object.setPrototypeOf(result, WebAudioWaveShaper.prototype);
            connect(node, result);
            return result;
        }
        return _this;
    }
    /**
     * @return {?}
     */
    WebAudioWaveShaper.prototype.ngOnDestroy = /**
     * @return {?}
     */
    function () {
        this.disconnect();
    };
    WebAudioWaveShaper.decorators = [
        { type: Directive, args: [{
                    selector: '[waWaveShaperNode]',
                    exportAs: 'AudioNode',
                    inputs: [
                        'oversample',
                        'curve',
                        'channelCount',
                        'channelCountMode',
                        'channelInterpretation',
                    ],
                    providers: [
                        {
                            provide: AUDIO_NODE,
                            useExisting: forwardRef(function () { return WebAudioWaveShaper; }),
                        },
                    ],
                },] }
    ];
    /** @nocollapse */
    WebAudioWaveShaper.ctorParameters = function () { return [
        { type: BaseAudioContext, decorators: [{ type: Inject, args: [AUDIO_CONTEXT,] }] },
        { type: undefined, decorators: [{ type: SkipSelf }, { type: Inject, args: [AUDIO_NODE,] }] },
        { type: Boolean, decorators: [{ type: Inject, args: [CONSTRUCTOR_SUPPORT,] }] }
    ]; };
    return WebAudioWaveShaper;
}(WaveShaperNode));

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
// @dynamic
var WebAudioWorklet = /** @class */ (function (_super) {
    __extends(WebAudioWorklet, _super);
    function WebAudioWorklet(context, node, name) {
        var _this = _super.call(this, context, name) || this;
        _this.processorerror = new EventEmitter();
        _this.onprocessorerror = function () { return _this.processorerror.emit(); };
        connect(node, _this);
        return _this;
    }
    /**
     * @return {?}
     */
    WebAudioWorklet.prototype.ngOnDestroy = /**
     * @return {?}
     */
    function () {
        this.disconnect();
    };
    WebAudioWorklet.decorators = [
        { type: Directive, args: [{
                    selector: '[waAudioWorkletNode][name]',
                    exportAs: 'AudioNode',
                    inputs: ['channelCount', 'channelCountMode', 'channelInterpretation'],
                    providers: [
                        {
                            provide: AUDIO_NODE,
                            useExisting: forwardRef(function () { return WebAudioWorklet; }),
                        },
                    ],
                },] }
    ];
    /** @nocollapse */
    WebAudioWorklet.ctorParameters = function () { return [
        { type: BaseAudioContext, decorators: [{ type: Inject, args: [AUDIO_CONTEXT,] }] },
        { type: undefined, decorators: [{ type: SkipSelf }, { type: Inject, args: [AUDIO_NODE,] }] },
        { type: String, decorators: [{ type: Attribute, args: ['name',] }] }
    ]; };
    WebAudioWorklet.propDecorators = {
        processorerror: [{ type: Output }]
    };
    return WebAudioWorklet;
}(AudioWorkletNode));

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
var WebAudioParamPipe = /** @class */ (function () {
    function WebAudioParamPipe() {
    }
    /**
     * Creates {@link AudioParamAutomation} to use with {@link AudioParam} inputs
     *
     * @param value target value
     * @param duration duration of the automation
     * @param mode either instant for given duration or linear/exponential ramp
     */
    /**
     * Creates {\@link AudioParamAutomation} to use with {\@link AudioParam} inputs
     *
     * @param {?} value target value
     * @param {?} duration duration of the automation
     * @param {?=} mode either instant for given duration or linear/exponential ramp
     * @return {?}
     */
    WebAudioParamPipe.prototype.transform = /**
     * Creates {\@link AudioParamAutomation} to use with {\@link AudioParam} inputs
     *
     * @param {?} value target value
     * @param {?} duration duration of the automation
     * @param {?=} mode either instant for given duration or linear/exponential ramp
     * @return {?}
     */
    function (value, duration, mode) {
        if (mode === void 0) { mode = 'exponential'; }
        return value instanceof Array
            ? {
                value: value,
                duration: duration,
            }
            : {
                value: value,
                duration: duration,
                mode: mode,
            };
    };
    WebAudioParamPipe.decorators = [
        { type: Pipe, args: [{
                    name: 'waAudioParam',
                },] }
    ];
    return WebAudioParamPipe;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
// @dynamic
var WebAudioPeriodicWavePipe = /** @class */ (function () {
    function WebAudioPeriodicWavePipe(context) {
        this.context = context;
    }
    /**
     * Creates {@link PeriodicWave} to use with {@link OscillatorNode}
     *
     * @param real cosine terms (traditionally the A terms)
     * @param imag sine terms (traditionally the B terms)
     * @param disableNormalization see {@lin PeriodicWaveConstraints}
     */
    /**
     * Creates {\@link PeriodicWave} to use with {\@link OscillatorNode}
     *
     * @param {?} real cosine terms (traditionally the A terms)
     * @param {?=} imag sine terms (traditionally the B terms)
     * @param {?=} disableNormalization see {\@lin PeriodicWaveConstraints}
     * @return {?}
     */
    WebAudioPeriodicWavePipe.prototype.transform = /**
     * Creates {\@link PeriodicWave} to use with {\@link OscillatorNode}
     *
     * @param {?} real cosine terms (traditionally the A terms)
     * @param {?=} imag sine terms (traditionally the B terms)
     * @param {?=} disableNormalization see {\@lin PeriodicWaveConstraints}
     * @return {?}
     */
    function (real, imag, disableNormalization) {
        return this.context.createPeriodicWave(new Float32Array(real), imag ? new Float32Array(imag) : new Float32Array(real.length), {
            disableNormalization: !!disableNormalization,
        });
    };
    WebAudioPeriodicWavePipe.decorators = [
        { type: Pipe, args: [{
                    name: 'waPeriodicWave',
                },] }
    ];
    /** @nocollapse */
    WebAudioPeriodicWavePipe.ctorParameters = function () { return [
        { type: BaseAudioContext, decorators: [{ type: Inject, args: [AUDIO_CONTEXT,] }] }
    ]; };
    return WebAudioPeriodicWavePipe;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
// @dynamic
var WebAudioBufferSource = /** @class */ (function (_super) {
    __extends(WebAudioBufferSource, _super);
    function WebAudioBufferSource(audioBufferService, context, modern, autoplay, detuneArg, playbackRateArg) {
        var _this = this;
        /** @type {?} */
        var detune = parse(detuneArg, 0);
        /** @type {?} */
        var playbackRate = parse(playbackRateArg, 1);
        if (modern) {
            _this = _super.call(this, context, { detune: detune, playbackRate: playbackRate }) || this;
            WebAudioBufferSource.init(_this, null, autoplay, audioBufferService);
        }
        else {
            /** @type {?} */
            var result = (/** @type {?} */ (context.createBufferSource()));
            Object.setPrototypeOf(WebAudioBufferSource.prototype, Object.getPrototypeOf(result));
            Object.setPrototypeOf(result, WebAudioBufferSource.prototype);
            result.playbackRate.value = playbackRate;
            WebAudioBufferSource.init(result, null, autoplay, audioBufferService);
            return result;
        }
        return _this;
    }
    Object.defineProperty(WebAudioBufferSource.prototype, "bufferSetter", {
        set: /**
         * @param {?} source
         * @return {?}
         */
        function (source) {
            this.buffer$.next(source);
        },
        enumerable: true,
        configurable: true
    });
    /**
     * @return {?}
     */
    WebAudioBufferSource.prototype.ngOnDestroy = /**
     * @return {?}
     */
    function () {
        this.buffer$.complete();
        try {
            this.stop();
        }
        catch (_) { }
        this.disconnect();
    };
    /**
     * @param {?} that
     * @param {?} _node
     * @param {?} autoplay
     * @param {?} audioBufferService
     * @return {?}
     */
    WebAudioBufferSource.init = /**
     * @param {?} that
     * @param {?} _node
     * @param {?} autoplay
     * @param {?} audioBufferService
     * @return {?}
     */
    function (that, _node, autoplay, audioBufferService) {
        if (autoplay !== null) {
            that.start();
        }
        /** @type {?} */
        var ended = new EventEmitter();
        that.ended = ended;
        that.onended = function () { return ended.emit(); };
        that.buffer$ = new Subject();
        that.buffer$
            .pipe(switchMap(function (source) {
            return typeof source === 'string'
                ? audioBufferService.fetch(source)
                : of(source);
        }))
            .subscribe(function (buffer) {
            that.buffer = buffer;
        });
    };
    WebAudioBufferSource.decorators = [
        { type: Directive, args: [{
                    selector: '[waAudioBufferSourceNode]',
                    exportAs: 'AudioNode',
                    inputs: [
                        'loop',
                        'loopStart',
                        'loopEnd',
                        'channelCount',
                        'channelCountMode',
                        'channelInterpretation',
                    ],
                    providers: [
                        {
                            provide: AUDIO_NODE,
                            useExisting: forwardRef(function () { return WebAudioBufferSource; }),
                        },
                    ],
                },] }
    ];
    /** @nocollapse */
    WebAudioBufferSource.ctorParameters = function () { return [
        { type: AudioBufferService, decorators: [{ type: Inject, args: [AudioBufferService,] }] },
        { type: BaseAudioContext, decorators: [{ type: Inject, args: [AUDIO_CONTEXT,] }] },
        { type: Boolean, decorators: [{ type: Inject, args: [CONSTRUCTOR_SUPPORT,] }] },
        { type: undefined, decorators: [{ type: Attribute, args: ['autoplay',] }] },
        { type: undefined, decorators: [{ type: Attribute, args: ['detune',] }] },
        { type: undefined, decorators: [{ type: Attribute, args: ['playbackRate',] }] }
    ]; };
    WebAudioBufferSource.propDecorators = {
        bufferSetter: [{ type: Input, args: ['buffer',] }],
        detuneParam: [{ type: Input, args: ['detune',] }],
        playbackRateParam: [{ type: Input, args: ['playbackRate',] }],
        ended: [{ type: Output }]
    };
    __decorate([
        audioParam('detune'),
        __metadata("design:type", Object)
    ], WebAudioBufferSource.prototype, "detuneParam", void 0);
    __decorate([
        audioParam('playbackRate'),
        __metadata("design:type", Object)
    ], WebAudioBufferSource.prototype, "playbackRateParam", void 0);
    return WebAudioBufferSource;
}(AudioBufferSourceNode));

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
// @dynamic
var WebAudioConstantSource = /** @class */ (function (_super) {
    __extends(WebAudioConstantSource, _super);
    function WebAudioConstantSource(context, autoplay, offset) {
        var _this = _super.call(this, context, {
            offset: parse(offset, 0),
        }) || this;
        _this.ended = new EventEmitter();
        _this.onended = function () { return _this.ended.emit(); };
        if (autoplay !== null) {
            _this.start();
        }
        return _this;
    }
    /**
     * @return {?}
     */
    WebAudioConstantSource.prototype.ngOnDestroy = /**
     * @return {?}
     */
    function () {
        try {
            this.stop();
        }
        catch (_) { }
        this.disconnect();
    };
    WebAudioConstantSource.decorators = [
        { type: Directive, args: [{
                    selector: '[waConstantSourceNode]',
                    exportAs: 'AudioNode',
                    inputs: ['channelCount', 'channelCountMode', 'channelInterpretation'],
                    providers: [
                        {
                            provide: AUDIO_NODE,
                            useExisting: forwardRef(function () { return WebAudioConstantSource; }),
                        },
                    ],
                },] }
    ];
    /** @nocollapse */
    WebAudioConstantSource.ctorParameters = function () { return [
        { type: BaseAudioContext, decorators: [{ type: Inject, args: [AUDIO_CONTEXT,] }] },
        { type: undefined, decorators: [{ type: Attribute, args: ['autoplay',] }] },
        { type: undefined, decorators: [{ type: Attribute, args: ['offset',] }] }
    ]; };
    WebAudioConstantSource.propDecorators = {
        offsetParam: [{ type: Input, args: ['offset',] }],
        ended: [{ type: Output }]
    };
    __decorate([
        audioParam('offset'),
        __metadata("design:type", Object)
    ], WebAudioConstantSource.prototype, "offsetParam", void 0);
    return WebAudioConstantSource;
}(ConstantSourceNode));

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
// @dynamic
var WebAudioMediaSource = /** @class */ (function (_super) {
    __extends(WebAudioMediaSource, _super);
    function WebAudioMediaSource(context, modern, _a) {
        var nativeElement = _a.nativeElement;
        var _this = this;
        if (modern) {
            _this = _super.call(this, context, { mediaElement: nativeElement }) || this;
        }
        else {
            /** @type {?} */
            var result = context.createMediaElementSource(nativeElement);
            Object.setPrototypeOf(result, WebAudioMediaSource.prototype);
            return (/** @type {?} */ (result));
        }
        return _this;
    }
    /**
     * @return {?}
     */
    WebAudioMediaSource.prototype.ngOnDestroy = /**
     * @return {?}
     */
    function () {
        this.disconnect();
    };
    WebAudioMediaSource.decorators = [
        { type: Directive, args: [{
                    selector: 'audio[waMediaElementAudioSourceNode], video[waMediaElementAudioSourceNode]',
                    exportAs: 'AudioNode',
                    providers: [
                        {
                            provide: AUDIO_NODE,
                            useExisting: forwardRef(function () { return WebAudioMediaSource; }),
                        },
                    ],
                },] }
    ];
    /** @nocollapse */
    WebAudioMediaSource.ctorParameters = function () { return [
        { type: AudioContext, decorators: [{ type: Inject, args: [AUDIO_CONTEXT,] }] },
        { type: Boolean, decorators: [{ type: Inject, args: [CONSTRUCTOR_SUPPORT,] }] },
        { type: ElementRef, decorators: [{ type: Inject, args: [ElementRef,] }] }
    ]; };
    return WebAudioMediaSource;
}(MediaElementAudioSourceNode));

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/** @type {?} */
var MEDIA_STREAM = new InjectionToken('MediaStream for MediaStreamAudioSourceNode');

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
// @dynamic
var WebAudioMediaStreamSource = /** @class */ (function (_super) {
    __extends(WebAudioMediaStreamSource, _super);
    function WebAudioMediaStreamSource(mediaStream, context, modern) {
        var _this = this;
        if (modern) {
            _this = _super.call(this, context, { mediaStream: mediaStream }) || this;
        }
        else {
            /** @type {?} */
            var result = context.createMediaStreamSource(mediaStream);
            Object.setPrototypeOf(result, WebAudioMediaStreamSource.prototype);
            return (/** @type {?} */ (result));
        }
        return _this;
    }
    /**
     * @return {?}
     */
    WebAudioMediaStreamSource.prototype.ngOnDestroy = /**
     * @return {?}
     */
    function () {
        this.disconnect();
    };
    WebAudioMediaStreamSource.decorators = [
        { type: Directive, args: [{
                    selector: '[waMediaStreamAudioSourceNode]',
                    exportAs: 'AudioNode',
                    providers: [
                        {
                            provide: AUDIO_NODE,
                            useExisting: forwardRef(function () { return WebAudioMediaStreamSource; }),
                        },
                    ],
                },] }
    ];
    /** @nocollapse */
    WebAudioMediaStreamSource.ctorParameters = function () { return [
        { type: MediaStream, decorators: [{ type: Inject, args: [MEDIA_STREAM,] }] },
        { type: AudioContext, decorators: [{ type: Inject, args: [AUDIO_CONTEXT,] }] },
        { type: Boolean, decorators: [{ type: Inject, args: [CONSTRUCTOR_SUPPORT,] }] }
    ]; };
    return WebAudioMediaStreamSource;
}(MediaStreamAudioSourceNode));

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/** @type {?} */
var MEDIA_STREAM_TRACK = new InjectionToken('MediaStreamTrack for MediaStreamTrackAudioSourceNode');

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
// @dynamic
var WebAudioMediaStreamTrackSource = /** @class */ (function (_super) {
    __extends(WebAudioMediaStreamTrackSource, _super);
    function WebAudioMediaStreamTrackSource(mediaStreamTrack, context) {
        return _super.call(this, context, { mediaStreamTrack: mediaStreamTrack }) || this;
    }
    /**
     * @return {?}
     */
    WebAudioMediaStreamTrackSource.prototype.ngOnDestroy = /**
     * @return {?}
     */
    function () {
        this.disconnect();
    };
    WebAudioMediaStreamTrackSource.decorators = [
        { type: Directive, args: [{
                    selector: '[waMediaStreamTrackAudioSourceNode]',
                    exportAs: 'AudioNode',
                    providers: [
                        {
                            provide: AUDIO_NODE,
                            useExisting: forwardRef(function () { return WebAudioMediaStreamTrackSource; }),
                        },
                    ],
                },] }
    ];
    /** @nocollapse */
    WebAudioMediaStreamTrackSource.ctorParameters = function () { return [
        { type: MediaStreamTrack, decorators: [{ type: Inject, args: [MEDIA_STREAM_TRACK,] }] },
        { type: AudioContext, decorators: [{ type: Inject, args: [AUDIO_CONTEXT,] }] }
    ]; };
    return WebAudioMediaStreamTrackSource;
}(MediaStreamTrackAudioSourceNode));

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
// @dynamic
var WebAudioOscillator = /** @class */ (function (_super) {
    __extends(WebAudioOscillator, _super);
    function WebAudioOscillator(context, modern, autoplay, detuneArg, frequencyArg) {
        var _this = this;
        /** @type {?} */
        var detune = parse(detuneArg, 0);
        /** @type {?} */
        var frequency = parse(frequencyArg, 440);
        if (modern) {
            _this = _super.call(this, context, { detune: detune, frequency: frequency }) || this;
            WebAudioOscillator.init(_this, null, autoplay);
        }
        else {
            /** @type {?} */
            var result = (/** @type {?} */ (context.createOscillator()));
            Object.setPrototypeOf(WebAudioOscillator.prototype, Object.getPrototypeOf(result));
            Object.setPrototypeOf(result, WebAudioOscillator.prototype);
            result.detune.value = detune;
            result.frequency.value = frequency;
            WebAudioOscillator.init(result, null, autoplay);
            return result;
        }
        return _this;
    }
    Object.defineProperty(WebAudioOscillator.prototype, "periodicWave", {
        set: /**
         * @param {?} periodicWave
         * @return {?}
         */
        function (periodicWave) {
            this.setPeriodicWave(periodicWave);
        },
        enumerable: true,
        configurable: true
    });
    /**
     * @return {?}
     */
    WebAudioOscillator.prototype.ngOnDestroy = /**
     * @return {?}
     */
    function () {
        try {
            this.stop();
        }
        catch (_) { }
        this.disconnect();
    };
    /**
     * @param {?} that
     * @param {?} node
     * @param {?} autoplay
     * @return {?}
     */
    WebAudioOscillator.init = /**
     * @param {?} that
     * @param {?} node
     * @param {?} autoplay
     * @return {?}
     */
    function (that, node, autoplay) {
        connect(node, that);
        if (autoplay !== null) {
            that.start();
        }
        /** @type {?} */
        var ended = new EventEmitter();
        that.ended = ended;
        that.onended = function () { return ended.emit(); };
    };
    WebAudioOscillator.decorators = [
        { type: Directive, args: [{
                    selector: '[waOscillatorNode]',
                    exportAs: 'AudioNode',
                    inputs: ['type', 'channelCount', 'channelCountMode', 'channelInterpretation'],
                    providers: [
                        {
                            provide: AUDIO_NODE,
                            useExisting: forwardRef(function () { return WebAudioOscillator; }),
                        },
                    ],
                },] }
    ];
    /** @nocollapse */
    WebAudioOscillator.ctorParameters = function () { return [
        { type: BaseAudioContext, decorators: [{ type: Inject, args: [AUDIO_CONTEXT,] }] },
        { type: Boolean, decorators: [{ type: Inject, args: [CONSTRUCTOR_SUPPORT,] }] },
        { type: undefined, decorators: [{ type: Attribute, args: ['autoplay',] }] },
        { type: undefined, decorators: [{ type: Attribute, args: ['detune',] }] },
        { type: undefined, decorators: [{ type: Attribute, args: ['frequency',] }] }
    ]; };
    WebAudioOscillator.propDecorators = {
        periodicWave: [{ type: Input }],
        detuneParam: [{ type: Input, args: ['detune',] }],
        frequencyParam: [{ type: Input, args: ['frequency',] }],
        ended: [{ type: Output }]
    };
    __decorate([
        audioParam('detune'),
        __metadata("design:type", Object)
    ], WebAudioOscillator.prototype, "detuneParam", void 0);
    __decorate([
        audioParam('frequency'),
        __metadata("design:type", Object)
    ], WebAudioOscillator.prototype, "frequencyParam", void 0);
    return WebAudioOscillator;
}(OscillatorNode));

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/** @type {?} */
var AUDIO_WORKLET_PROCESSORS = new InjectionToken('Web Audio API worklet processors paths', {
    providedIn: 'root',
    factory: function () { return []; },
});

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/** @type {?} */
var AUDIO_WORKLET_PROCESSORS_READY = new InjectionToken('Web Audio API worklet processors resolution promise', {
    providedIn: 'root',
    factory: function () {
        /** @type {?} */
        var context = inject(AUDIO_CONTEXT);
        /** @type {?} */
        var processors = inject(AUDIO_WORKLET_PROCESSORS);
        if (!context.audioWorklet) {
            return Promise.reject('AudioWorklet is not supported');
        }
        return Promise.all(processors.map(function (processor) { return context.audioWorklet.addModule(processor); })).then(function () { return true; });
    },
});

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/** @type {?} */
var WEB_AUDIO_SUPPORT = new InjectionToken('Web Audio API support', {
    providedIn: 'root',
    factory: function () { return !!AudioContext; },
});

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
var WebAudioModule = /** @class */ (function () {
    function WebAudioModule() {
    }
    WebAudioModule.decorators = [
        { type: NgModule, args: [{
                    declarations: [
                        WebAudioContext,
                        WebAudioChannel,
                        WebAudioDestination,
                        WebAudioListener,
                        WebAudioOfflineContext,
                        WebAudioOutput,
                        WebAudioMediaStreamDestination,
                        WebAudioBufferSource,
                        WebAudioConstantSource,
                        WebAudioMediaSource,
                        WebAudioMediaStreamSource,
                        WebAudioMediaStreamTrackSource,
                        WebAudioOscillator,
                        WebAudioAnalyser,
                        WebAudioBiquadFilter,
                        WebAudioChannelSplitter,
                        WebAudioChannelMerger,
                        WebAudioConvolver,
                        WebAudioDelay,
                        WebAudioDynamicsCompressor,
                        WebAudioGain,
                        WebAudioIIRFilter,
                        WebAudioPanner,
                        WebAudioScriptProcessor,
                        WebAudioStereoPanner,
                        WebAudioWaveShaper,
                        WebAudioWorklet,
                        WebAudioParamPipe,
                        WebAudioPeriodicWavePipe,
                    ],
                    exports: [
                        WebAudioContext,
                        WebAudioChannel,
                        WebAudioDestination,
                        WebAudioListener,
                        WebAudioOfflineContext,
                        WebAudioOutput,
                        WebAudioMediaStreamDestination,
                        WebAudioBufferSource,
                        WebAudioConstantSource,
                        WebAudioMediaSource,
                        WebAudioMediaStreamSource,
                        WebAudioMediaStreamTrackSource,
                        WebAudioOscillator,
                        WebAudioAnalyser,
                        WebAudioBiquadFilter,
                        WebAudioChannelSplitter,
                        WebAudioChannelMerger,
                        WebAudioConvolver,
                        WebAudioDelay,
                        WebAudioDynamicsCompressor,
                        WebAudioGain,
                        WebAudioIIRFilter,
                        WebAudioPanner,
                        WebAudioScriptProcessor,
                        WebAudioStereoPanner,
                        WebAudioWaveShaper,
                        WebAudioWorklet,
                        WebAudioParamPipe,
                        WebAudioPeriodicWavePipe,
                    ],
                },] }
    ];
    return WebAudioModule;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

export { audioParam, WebAudioContext, WebAudioChannel, WebAudioDestination, WebAudioListener, WebAudioOfflineContext, WebAudioOutput, WebAudioMediaStreamDestination, WebAudioAnalyser, WebAudioBiquadFilter, WebAudioChannelMerger, WebAudioChannelSplitter, WebAudioConvolver, WebAudioDelay, WebAudioDynamicsCompressor, WebAudioGain, WebAudioIIRFilter, WebAudioPanner, WebAudioScriptProcessor, WebAudioStereoPanner, WebAudioWaveShaper, WebAudioWorklet, WebAudioParamPipe, WebAudioPeriodicWavePipe, AudioBufferService, WebAudioBufferSource, WebAudioConstantSource, WebAudioMediaSource, WebAudioMediaStreamSource, WebAudioMediaStreamTrackSource, WebAudioOscillator, AUDIO_CONTEXT, AUDIO_NODE, AUDIO_WORKLET_PROCESSORS, AUDIO_WORKLET_PROCESSORS_READY, FEEDBACK_COEFFICIENTS, FEEDFORWARD_COEFFICIENTS, MEDIA_STREAM, MEDIA_STREAM_TRACK, WEB_AUDIO_SUPPORT, WebAudioModule, CONSTRUCTOR_SUPPORT as ɵa };

//# sourceMappingURL=ng-web-apis-audio.js.map